@@ -4,8 +4,10 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"reflect"
 	"time"
 
+	"github.com/openshift/elasticsearch-operator/pkg/elasticsearch"
 	"github.com/openshift/elasticsearch-operator/pkg/utils"
 	"github.com/openshift/elasticsearch-operator/pkg/utils/comparators"
 
@@ -37,9 +39,11 @@ type deploymentNode struct {
 	clusterSize int32
 
 	client client.Client
+
+	esClient elasticsearch.Client
 }
 
-func (deploymentNode *deploymentNode) populateReference(nodeName string, node api.ElasticsearchNode, cluster *api.Elasticsearch, roleMap map[api.ElasticsearchNodeRole]bool, replicas int32, client client.Client) {
+func (deploymentNode *deploymentNode) populateReference(nodeName string, node api.ElasticsearchNode, cluster *api.Elasticsearch, roleMap map[api.ElasticsearchNodeRole]bool, replicas int32, client client.Client, esClient elasticsearch.Client) {
 
 	labels := newLabels(cluster.Name, nodeName, roleMap)
 
@@ -56,7 +60,7 @@ func (deploymentNode *deploymentNode) populateReference(nodeName string, node ap
 	}
 
 	progressDeadlineSeconds := int32(1800)
-
+	logConfig := getLogConfig(cluster.GetAnnotations())
 	deployment.Spec = apps.DeploymentSpec{
 		Replicas: &replicas,
 		Selector: &metav1.LabelSelector{
@@ -67,7 +71,7 @@ func (deploymentNode *deploymentNode) populateReference(nodeName string, node ap
 		},
 		ProgressDeadlineSeconds: &progressDeadlineSeconds,
 		Paused:                  false,
-		Template:                newPodTemplateSpec(nodeName, cluster.Name, cluster.Namespace, node, cluster.Spec.Spec, labels, roleMap, client),
+		Template:                newPodTemplateSpec(nodeName, cluster.Name, cluster.Namespace, node, cluster.Spec.Spec, labels, roleMap, client, logConfig),
 	}
 
 	addOwnerRefToObject(&deployment, getOwnerRef(cluster))
@@ -76,6 +80,7 @@ func (deploymentNode *deploymentNode) populateReference(nodeName string, node ap
 	deploymentNode.clusterName = cluster.Name
 
 	deploymentNode.client = client
+	deploymentNode.esClient = esClient
 }
 
 func (current *deploymentNode) updateReference(node NodeTypeInterface) {
@@ -126,8 +131,8 @@ func (node *deploymentNode) state() api.ElasticsearchNodeStatus {
 	}
 }
 
-func (node *deploymentNode) delete() {
-	node.client.Delete(context.TODO(), &node.self)
+func (node *deploymentNode) delete() error {
+	return node.client.Delete(context.TODO(), &node.self)
 }
 
 func (node *deploymentNode) create() error {
@@ -281,13 +286,7 @@ func (node *deploymentNode) replicaCount() (error, int32) {
 
 func (node *deploymentNode) waitForNodeRejoinCluster() (error, bool) {
 	err := wait.Poll(time.Second*1, time.Second*60, func() (done bool, err error) {
-		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
-		if err != nil {
-			logrus.Warnf("Unable to get cluster size waiting for %v to rejoin cluster", node.name())
-			return false, getErr
-		}
-
-		return (node.clusterSize <= clusterSize), nil
+		return node.esClient.IsNodeInCluster(node.name())
 	})
 
 	return err, (err == nil)
@@ -295,13 +294,9 @@ func (node *deploymentNode) waitForNodeRejoinCluster() (error, bool) {
 
 func (node *deploymentNode) waitForNodeLeaveCluster() (error, bool) {
 	err := wait.Poll(time.Second*1, time.Second*60, func() (done bool, err error) {
-		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
-		if err != nil {
-			logrus.Warnf("Unable to get cluster size waiting for %v to leave cluster", node.name())
-			return false, getErr
-		}
+		inCluster, checkErr := node.esClient.IsNodeInCluster(node.name())
 
-		return (node.clusterSize > clusterSize), nil
+		return !inCluster, checkErr
 	})
 
 	return err, (err == nil)
@@ -321,12 +316,12 @@ func (node *deploymentNode) isMissing() bool {
 func (node *deploymentNode) rollingRestart(upgradeStatus *api.ElasticsearchNodeStatus) {
 
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
-		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); !utils.Contains(desiredClusterStates, status) {
+		if status, _ := node.esClient.GetClusterHealthStatus(); !utils.Contains(desiredClusterStates, status) {
 			logrus.Infof("Waiting for cluster to be recovered before restarting %s: %s / %v", node.name(), status, desiredClusterStates)
 			return
 		}
 
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
+		size, err := node.esClient.GetClusterNodeCount()
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to restart for %v", node.name())
 			return
@@ -346,12 +341,12 @@ func (node *deploymentNode) rollingRestart(upgradeStatus *api.ElasticsearchNodeS
 		if replicas > 0 {
 
 			// disable shard allocation
-			if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationPrimaries, node.client); !ok {
+			if ok, err := node.esClient.SetShardAllocation(api.ShardAllocationPrimaries); !ok {
 				logrus.Warnf("Unable to disable shard allocation: %v", err)
 				return
 			}
 
-			if ok, err := DoSynchronizedFlush(node.clusterName, node.self.Namespace, node.client); !ok {
+			if ok, err := node.esClient.DoSynchronizedFlush(); !ok {
 				logrus.Warnf("Unable to perform synchronized flush: %v", err)
 			}
 
@@ -376,7 +371,9 @@ func (node *deploymentNode) rollingRestart(upgradeStatus *api.ElasticsearchNodeS
 
 		// if the node doesn't exist -- create it
 		if node.isMissing() {
-			node.create()
+			if err := node.create(); err != nil {
+				logrus.Warnf("unable to create node. E: %s\r\n", err.Error())
+			}
 		}
 
 		if err := node.setReplicaCount(1); err != nil {
@@ -392,7 +389,7 @@ func (node *deploymentNode) rollingRestart(upgradeStatus *api.ElasticsearchNodeS
 		node.refreshHashes()
 
 		// reenable shard allocation
-		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationAll, node.client); !ok {
+		if ok, err := node.esClient.SetShardAllocation(api.ShardAllocationAll); !ok {
 			logrus.Warnf("Unable to enable shard allocation: %v", err)
 			return
 		}
@@ -402,7 +399,7 @@ func (node *deploymentNode) rollingRestart(upgradeStatus *api.ElasticsearchNodeS
 
 	if upgradeStatus.UpgradeStatus.UpgradePhase == api.RecoveringData {
 
-		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); !utils.Contains(desiredClusterStates, status) {
+		if status, _ := node.esClient.GetClusterHealthStatus(); !utils.Contains(desiredClusterStates, status) {
 			logrus.Infof("Waiting for cluster to recover: %s / %v", status, desiredClusterStates)
 			return
 		}
@@ -416,7 +413,7 @@ func (node *deploymentNode) fullClusterRestart(upgradeStatus *api.ElasticsearchN
 
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
 		upgradeStatus.UpgradeStatus.UnderUpgrade = v1.ConditionTrue
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
+		size, err := node.esClient.GetClusterNodeCount()
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to restart for %v", node.name())
 			return
@@ -473,12 +470,12 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 
 	// set our state to being under upgrade
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
-		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); !utils.Contains(desiredClusterStates, status) {
+		if status, _ := node.esClient.GetClusterHealthStatus(); !utils.Contains(desiredClusterStates, status) {
 			logrus.Infof("Waiting for cluster to be recovered before upgrading %s: %s / %v", node.name(), status, desiredClusterStates)
 			return fmt.Errorf("Cluster not in at least %s state before beginning upgrade: %s", yellowClusterState, status)
 		}
 
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
+		size, err := node.esClient.GetClusterNodeCount()
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to update for %v", node.name())
 		}
@@ -491,12 +488,12 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 		upgradeStatus.UpgradeStatus.UpgradePhase == api.ControllerUpdated {
 
 		// disable shard allocation
-		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationPrimaries, node.client); !ok {
+		if ok, err := node.esClient.SetShardAllocation(api.ShardAllocationPrimaries); !ok {
 			logrus.Warnf("Unable to disable shard allocation: %v", err)
 			return err
 		}
 
-		if ok, err := DoSynchronizedFlush(node.clusterName, node.self.Namespace, node.client); !ok {
+		if ok, err := node.esClient.DoSynchronizedFlush(); !ok {
 			logrus.Warnf("Unable to perform synchronized flush: %v", err)
 		}
 
@@ -511,7 +508,9 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 	if upgradeStatus.UpgradeStatus.UpgradePhase == api.NodeRestarting {
 
 		// do a unpause, wait, and pause again
-		node.unpause()
+		if err := node.unpause(); err != nil {
+			logrus.Warnf("unable to unpause node. E: %s\r\n", err.Error())
+		}
 
 		// wait for rollout
 		if err := node.waitForNodeRollout(node.currentRevision); err != nil {
@@ -520,7 +519,9 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 		}
 
 		// pause again
-		node.pause()
+		if err := node.pause(); err != nil {
+			logrus.Warnf("unable to pause node. E: %s\r\n", err.Error())
+		}
 
 		// once we've restarted this is taken care of
 		node.refreshHashes()
@@ -532,7 +533,7 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 		}
 
 		// reenable shard allocation
-		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationAll, node.client); !ok {
+		if ok, err := node.esClient.SetShardAllocation(api.ShardAllocationAll); !ok {
 			logrus.Warnf("Unable to enable shard allocation: %v", err)
 			return err
 		}
@@ -542,7 +543,7 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 
 	if upgradeStatus.UpgradeStatus.UpgradePhase == api.RecoveringData {
 
-		if status, err := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); !utils.Contains(desiredClusterStates, status) {
+		if status, err := node.esClient.GetClusterHealthStatus(); !utils.Contains(desiredClusterStates, status) {
 			logrus.Infof("Waiting for cluster to recover: %s / %v", status, desiredClusterStates)
 			return err
 		}
@@ -575,6 +576,33 @@ func (node *deploymentNode) executeUpdate() error {
 	})
 }
 
+func (node *deploymentNode) progressNodeChanges(upgradeStatus *api.ElasticsearchNodeStatus) error {
+	if node.isChanged() {
+		if err := node.executeUpdate(); err != nil {
+			return err
+		}
+
+		node.currentRevision = node.nodeRevision()
+
+		if err := node.unpause(); err != nil {
+			return fmt.Errorf("unable to unpause node %q: %v", node.name(), err)
+		}
+
+		logrus.Debugf("Waiting for node '%s' to rollout...", node.name())
+
+		if err := node.waitForNodeRollout(node.currentRevision); err != nil {
+			return fmt.Errorf("Timed out waiting for node %v to rollout", node.name())
+		}
+
+		if err := node.pause(); err != nil {
+			return fmt.Errorf("unable to pause node %q: %v", node.name(), err)
+		}
+
+		node.refreshHashes()
+	}
+	return nil
+}
+
 func (node *deploymentNode) progressUnshedulableNode(upgradeStatus *api.ElasticsearchNodeStatus) error {
 	if node.isChanged() {
 		logrus.Infof("Requested to update node '%s', which is unschedulable. Skipping rolling restart scenario and performing redeploy now", upgradeStatus.DeploymentName)
@@ -586,8 +614,12 @@ func (node *deploymentNode) progressUnshedulableNode(upgradeStatus *api.Elastics
 		if err := node.unpause(); err != nil {
 			return err
 		}
-		// if unpause is succesfull, always try to pause
-		defer node.pause()
+		// if unpause is successful, always try to pause
+		defer func() {
+			if err := node.pause(); err != nil {
+				logrus.Warnf("unable to unpause node. E: %s\r\n", err.Error())
+			}
+		}()
 
 		logrus.Debugf("Waiting for node '%s' to rollout...", node.name())
 
@@ -642,7 +674,6 @@ func (node *deploymentNode) isChanged() bool {
 		node.self.Spec.Template.Spec.Tolerations = desired.Spec.Template.Spec.Tolerations
 		changed = true
 	}
-
 	// Only Image and Resources (CPU & memory) differences trigger rolling restart
 	for index := 0; index < len(node.self.Spec.Template.Spec.Containers); index++ {
 		nodeContainer := node.self.Spec.Template.Spec.Containers[index]
@@ -654,16 +685,27 @@ func (node *deploymentNode) isChanged() bool {
 			changed = true
 		}
 
+		if !comparators.EnvValueEqual(desiredContainer.Env, nodeContainer.Env) {
+			nodeContainer.Env = desiredContainer.Env
+			logger.Debugf("Setting Container %q EnvVars to desired: %v", nodeContainer.Name, nodeContainer.Env)
+			changed = true
+		}
+		if !reflect.DeepEqual(desiredContainer.Args, nodeContainer.Args) {
+			nodeContainer.Args = desiredContainer.Args
+			logger.Debugf("Container Args are different between current and desired for %s",
+				nodeContainer.Name)
+			changed = true
+		}
+		if !reflect.DeepEqual(desiredContainer.Ports, nodeContainer.Ports) {
+			nodeContainer.Ports = desiredContainer.Ports
+			logger.Debugf("Container Ports are different between current and desired for %s", nodeContainer.Name)
+			changed = true
+		}
 		var updatedContainer v1.Container
 		var resourceUpdated bool
 		if updatedContainer, resourceUpdated = updateResources(node, nodeContainer, desiredContainer); resourceUpdated {
 			changed = true
 		}
-		if !comparators.EnvValueEqual(desiredContainer.Env, nodeContainer.Env) {
-			updatedContainer.Env = desiredContainer.Env
-			logger.Debugf("Setting Container %q EnvVars to desired: %v", updatedContainer.Name, updatedContainer.Env)
-			changed = true
-		}
 
 		node.self.Spec.Template.Spec.Containers[index] = updatedContainer
 	}
